<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

class WC_Viapay_Order
{
	public function __construct() {
		add_filter( 'viapay_order_handle' , array( $this, 'get_order_handle' ), 10, 3 );
		add_filter( 'viapay_get_order' , array( $this, 'get_orderid_by_handle' ), 10, 2 );
	}

	/**
	 * Get Viapay Order Handle.
	 *
	 * @param string $handle
	 * @param mixed $order_id
	 * @param WC_Order $order
	 *
	 * @return mixed|string
	 */
	public function get_order_handle( $handle, $order_id, $order ) {
		$handle = get_post_meta( $order->get_id(), '_viapay_order', TRUE );
		if ( empty( $handle ) ) {
			$handle = 'order-' . $order->get_id();
			update_post_meta( $order->get_id(), '_viapay_order', $handle );
		}

		return $handle;
	}

	/**
	 * Get Order Id by Viapay Order Handle.
	 *
	 * @param int|null $order_id
	 * @param string $handle
	 *
	 * @return bool|mixed
	 */
	public function get_orderid_by_handle( $order_id, $handle ) {
		global $wpdb;

		$query = "
			SELECT post_id FROM {$wpdb->prefix}postmeta 
			LEFT JOIN {$wpdb->prefix}posts ON ({$wpdb->prefix}posts.ID = {$wpdb->prefix}postmeta.post_id)
			WHERE meta_key = %s AND meta_value = %s;";
		$sql = $wpdb->prepare( $query, '_viapay_order', $handle );
		$order_id = $wpdb->get_var( $sql );
		if ( ! $order_id ) {
			return false;
		}

		return $order_id;
	}
}

new WC_Viapay_Order();

